<?php

namespace Sma\Transaction\Http\Controllers;

use App\Http\Controllers\AdminController as BaseController;
use Form;
use Sma\Transaction\Http\Requests\TransactionRequest;
use Sma\Transaction\Interfaces\TransactionRepositoryInterface;
use Sma\Transaction\Models\Transaction;

/**
 * Admin web controller class.
 */
class TransactionAdminController extends BaseController
{
    // use TransactionWorkflow;
    /**
     * Initialize transaction controller.
     *
     * @param type TransactionRepositoryInterface $transaction
     *
     * @return type
     */
    public function __construct(TransactionRepositoryInterface $transaction)
    {
        $this->repository = $transaction;
        parent::__construct();
    }

    /**
     * Display a list of transaction.
     *
     * @return Response
     */
    public function index(TransactionRequest $request)
    {
        if ($request->wantsJson()) {
            return $this->getJson($request);
        }
        $this   ->theme->prependTitle(trans('transaction::transaction.names').' :: ');
        return $this->theme->of('transaction::admin.transaction.index')->render();
    }

    /**
     * Display a list of transaction.
     *
     * @return Response
     */
    public function getJson(TransactionRequest $request)
    {
        $pageLimit = $request->input('pageLimit');

        $transactions  = $this->repository
                ->pushCriteria(app('Litepie\Repository\Criteria\RequestCriteria'))
                ->setPresenter('\\Sma\\Transaction\\Repositories\\Presenter\\TransactionListPresenter')
                ->scopeQuery(function($query){
                    return $query->orderBy('id','DESC');
                })->paginate($pageLimit);
        $transactions['recordsTotal']    = $transactions['meta']['pagination']['total'];
        $transactions['recordsFiltered'] = $transactions['meta']['pagination']['total'];
        $transactions['request']         = $request->all();
        return response()->json($transactions, 200);

    }

    /**
     * Display transaction.
     *
     * @param Request $request
     * @param Model   $transaction
     *
     * @return Response
     */
    public function show(TransactionRequest $request, Transaction $transaction)
    {
        if (!$transaction->exists) {
            return response()->view('transaction::admin.transaction.new', compact('transaction'));
        }

        Form::populate($transaction);
        return response()->view('transaction::admin.transaction.show', compact('transaction'));
    }

    /**
     * Show the form for creating a new transaction.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TransactionRequest $request)
    {

        $transaction = $this->repository->newInstance([]);

        Form::populate($transaction);

        return response()->view('transaction::admin.transaction.create', compact('transaction'));

    }

    /**
     * Create new transaction.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TransactionRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.web');
            $transaction          = $this->repository->create($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('transaction::transaction.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/transaction/transaction/'.$transaction->getRouteKey())
            ], 201);


        } catch (Exception $e) {
            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
            ], 400);
        }
    }

    /**
     * Show transaction for editing.
     *
     * @param Request $request
     * @param Model   $transaction
     *
     * @return Response
     */
    public function edit(TransactionRequest $request, Transaction $transaction)
    {
        Form::populate($transaction);
        return  response()->view('transaction::admin.transaction.edit', compact('transaction'));
    }

    /**
     * Update the transaction.
     *
     * @param Request $request
     * @param Model   $transaction
     *
     * @return Response
     */
    public function update(TransactionRequest $request, Transaction $transaction)
    {
        try {

            $attributes = $request->all();

            $transaction->update($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('transaction::transaction.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/transaction/transaction/'.$transaction->getRouteKey())
            ], 201);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/transaction/transaction/'.$transaction->getRouteKey()),
            ], 400);

        }
    }

    /**
     * Remove the transaction.
     *
     * @param Model   $transaction
     *
     * @return Response
     */
    public function destroy(TransactionRequest $request, Transaction $transaction)
    {

        try {

            $t = $transaction->delete();

            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('transaction::transaction.name')]),
                'code'     => 202,
                'redirect' => trans_url('/admin/transaction/transaction/0'),
            ], 202);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/transaction/transaction/'.$transaction->getRouteKey()),
            ], 400);
        }
    }

}
